/*
This is a part of the LiteStep Shell Source code.

Copyright (C) 1997-2001 The LiteStep Development Team

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/
#ifndef __TASKBAR_H
#define __TASKBAR_H

#include <string>
#include <vector>
using namespace std;

class Taskbar;
class TaskbarButton;
class TaskbarSkin;
class Texture;

typedef vector<TaskbarButton *>::iterator TaskbarButtonIterator;

class Taskbar
{
private:   // variables

	HWND hWnd;
	HINSTANCE hInstance;
	HWND hToolTips;

	boolean alwaysOnTop;
	boolean visible;

	//boolean keepDesktopArea;

	int screenHeight;
	int screenWidth;

	RECT adjustedDeskArea;
	RECT prevDeskArea;

	BOOL active;
	//BOOL autoHidden;
	BOOL mouseOver;
	TaskbarButton *capture;

	BOOL startButton;
	BOOL startMousePressed;
	BOOL startMouseOver;

	BOOL isTaskFlash;


	TaskbarSkin *skin;
	TaskbarSkin *buttonSkin;
	TaskbarSkin *activeButtonSkin;
	TaskbarSkin *flashSkin;

	HFONT activeFont;
	HFONT font;
	HFONT minimizedFont;
	HFONT flashFont;

	vector<TaskbarButton *> buttons;

	boolean listChanged;

    int maxTaskWidth;
	int borderLeft;
	int borderTop;
	int borderRight;
	int borderBottom;

	int iconSize;
	//int iconSpacing;
	int spacing;

	int buttonBorderLeft;
	int buttonBorderTop;
	int buttonBorderRight;
	int buttonBorderBottom;

	int taskbarHeight;

    TaskbarSkin *minimizedSkin;
public:

	Taskbar();
	virtual ~Taskbar();

	void OnLoad( HINSTANCE );
	void OnUnload();

	void Invalidate( int, int, int, int );

	TaskbarButton *GetCapture();
	void SetCapture( TaskbarButton * );
	void ReleaseCapture();

	friend class TaskbarButton;

	//TRUE if the user specifies tasks flashing
	BOOL IsTaskFlash() const
	{
		return isTaskFlash;
	}

	boolean isAlwaysOnTop() const
	{
		return alwaysOnTop;
	}
	boolean isVisible() const
	{
		return visible;
	}

	//void setAlwaysOnTop(boolean alwaysOnTop);

	void hide();
	void show();
    BOOL IsAlreadyInList(HWND );
	BOOL showIcon;
	int savedWidth;
	int savedX;

	void Get1To4Settings(char *, int &, int &, int &, int &, int, int, int, int);
//	int taskbarTrayResize;
	//buggy
	//bool buttonFlashState;

	//bool deactivateLastActive;

	BOOL taskbarPositioning;
	int edge;

	int taskbarX;
	int taskbarY;
	int taskbarXS;
	int taskbarYS;

	HWND lastActive;
	//int lastActiveCount;
    int taskbarLines;
	void Layout();
	int neededHeight;
	void OnMove(int, int);
	int height;
	int width;

	BOOL reallyHidden;


    char taskbarLButtonDown[MAX_LINE_LENGTH];
    char taskbarRButtonDown[MAX_LINE_LENGTH];
    char taskbarLButtonUp[MAX_LINE_LENGTH];
    char taskbarRButtonUp[MAX_LINE_LENGTH];
    char taskbarMButtonDown[MAX_LINE_LENGTH];
    char taskbarMButtonUp[MAX_LINE_LENGTH];

	int TaskBarTileLButton;
	int TaskBarTileMButton;
	int TaskBarTileRButton;
	HWND getHwnd();
	bool waitForBox;
	int flashCount;
	bool scrollWas;
	bool inCriticalProcess;
	int timerRefresh;
	int timerScroll;
	bool timerScrollEnabled;

	bool AddWindowToList(HWND);
    void FillList(char * , vector<string>);
	int GetRCMouse(const char * , int);
	void GetRCList(char *, vector<string> &);
	bool LsBoxed;

	int iconX;
	int iconY;

	int buttonTextBorderLeft;
	int buttonTextBorderTop;
	int buttonTextBorderRight;
	int buttonTextBorderBottom;

	BOOL showText;
	int textX;
	int textY;

	int direction;

private:   // functions

	TaskbarButton *SearchForButtonByHandle( HWND );

	TaskbarButton *ButtonAtPoint( int, int );
	void ReadConfig();
	void SetWindowPosition();

	void Dock( HWND );

private:   // configuration variables

	//BOOL autoHide;
	//int autoHideDelay;

	BOOL stripTaskbar;

	BOOL msTaskbar;
	COLORREF foreColor1;
	COLORREF foreColor2;
	COLORREF backColor;
	COLORREF flashColor;
	COLORREF minColor;
	COLORREF activeColor;

	COLORREF textColor;
	COLORREF textActiveColor;
	COLORREF textMinimizedColor;
	COLORREF textFlashColor;


	BOOL noSkinShift;
	BOOL noTextShift;

private:   // message handlers

	void OnActivate( BOOL );
	void OnDisplayChange();
	void OnGetMinRect( HWND, RECT * );
	void OnLButtonDown( int, int );
	void OnMButtonDown( int, int );
	void OnMButtonUp( int, int );
	void OnLButtonUp( int, int );
	void OnMouseMove( int, int );
	void OnPaint( HDC );
	void OnRedraw( HWND, BOOL );
	void OnRButtonDown( int, int );
	void OnRButtonUp( int, int );
	void OnSettingChange();
	void OnSize( int, int );
	//void OnTimerAutoHide();
	void OnTimerUpdate(int);
	void OnWindowActivated( HWND, BOOL );
	void OnWindowCreated( HWND );
	void OnWindowDestroyed( HWND );
	BOOL OnWindowMessage( UINT, WPARAM, LPARAM, LRESULT & );

private:   // statics
	//char startButtonImage[256];
	//TaskbarSkin* startButtonSkin;
	//TaskbarButton* pStartButton;
	//char startButtonText[256];
	//int startButtonSize;

	static BOOL WINAPI EnumWindowsProc( HWND, LPARAM );
	static LRESULT WINAPI WndProc( HWND, UINT, WPARAM, LPARAM );
protected:
};

class TaskbarButton
{
private:

	boolean active;
	HICON icon;

	char* otherCaption;
	TaskbarSkin* skin;
	TaskbarSkin* activeskin;
	TaskbarSkin* flashskin;
	TaskbarSkin* minimizedskin;

	HWND hTask;

	LPSTR caption;

	BOOL mousePressed;
	BOOL mouseOver;
	BOOL taskflash; //current state

	int x;
	int y;
	int height;
	int width;

	Taskbar &container;

public:
	//jesus
	void Deactivate();
	//jesus
	void SetCaption(LPCSTR cap);
	TaskbarSkin* GetSkin();
	TaskbarSkin* GetActiveSkin();
	TaskbarSkin* GetFlashSkin();
	TaskbarSkin* GetMinimizedSkin();
	void SetSkin(TaskbarSkin *s);
	void SetActiveSkin(TaskbarSkin *s);
	void SetFlashSkin(TaskbarSkin *s);
	void SetMinimizedSkin(TaskbarSkin *s);

	TaskbarButton( Taskbar &, HWND );
	virtual ~TaskbarButton();

	BOOL HasPoint( int, int );
	void Move( int, int, int, int );

	void OnLButtonDown( int, int );
	void OnMButtonDown( int, int );
	void OnMButtonUp( int, int );
	bool isOnTop;

	void Close();
	void Close2();
    void SelectUp();
    void SelectDown();
	void MenuUp();
	void MenuDown();
	void OnTopToggle();
	void OnLButtonUp( int, int );
	void OnMouseMove( int, int );
	void OnPaint( HDC );
	void OnRButtonDown( int, int );
	void OnRButtonUp( int, int );

	/**/
	BOOL flashing;
	/**/
	HWND buttonHandler();

	// redraw == TRUE to force ths button to update
	//			 FALSE to let the button decide whether to update or not
	void OnUpdate(BOOL needRedraw = FALSE);

	inline HWND GetTask() const
	{
		return hTask;
	}
	inline BOOL IsActive() const
	{
		return active ? TRUE : FALSE;
	}

	void SetFlashing(BOOL flash)
	{
		taskflash = flash;
	}

	BOOL GetFlashing() const
	{
		return taskflash;
	}

	//HWND buttonHandler();
private:

	LPSTR GetWindowCaption();
	HICON GetIcon();

}
;

class TaskbarSkin
{
public:
	COLORREF GetBackColor();
	void SetBackColor(COLORREF color);
	BOOL stretch;
	BOOL GetStretch();
	void SetStretch(BOOL s);

	TaskbarSkin();
	TaskbarSkin( LPCSTR, LPCSTR, LPCSTR );
	virtual ~TaskbarSkin();

	void Apply( HDC, int, int, int, int );

protected:

	HBITMAP hbmLeft;
	int leftH;
	int leftW;

	HBITMAP hbmMiddle;
	int middleH;
	int middleW;

	HBITMAP hbmRight;
	int rightH;
	int rightW;
private:
	COLORREF backColor;
};

HFONT GetRCFont(const char *name, const LOGFONT *lfDefault);
HICON GetIconFromWindow( HWND, BOOL );
BOOL IsAppWindow( HWND );
BOOL WINAPIV PrintLog( LPCSTR, ... );

extern void (WINAPI *SwitchToThisWindow)( HWND, int );

#endif // __TASKBAR_H
